package com.myorg;

import java.util.HashMap;
import java.util.Map;

import software.amazon.awscdk.core.Construct;
import software.amazon.awscdk.core.Duration;
import software.amazon.awscdk.core.Stack;
import software.amazon.awscdk.core.StackProps;

import software.amazon.awscdk.services.lambda.Code;
import software.amazon.awscdk.services.lambda.Function;
import software.amazon.awscdk.services.lambda.Runtime;
import software.amazon.awscdk.services.lambda.StartingPosition;

import software.amazon.awscdk.services.lambda.nodejs.NodejsFunction;

import software.amazon.awscdk.services.apigateway.LambdaRestApi;

import software.amazon.awscdk.services.dynamodb.Attribute;
import software.amazon.awscdk.services.dynamodb.AttributeType;
import software.amazon.awscdk.services.dynamodb.Table;

import software.amazon.awscdk.services.lambda.eventsources.DynamoEventSourceProps;
import software.amazon.awscdk.services.lambda.eventsources.DynamoEventSource;

public class CreateCommandRestControllerStack extends Stack {
    public CreateCommandRestControllerStack(final Construct scope, final String id) {
        this(scope, id, null);
    }

    public CreateCommandRestControllerStack(final Construct scope, final String id, final StackProps props) {
        super(scope, id, props);

        // DynamoDB table
        final Attribute partitionKey = Attribute.builder()
                .name("id")
                .type(AttributeType.NUMBER)
                .build();
        

        final Table productTable = Table.Builder.create(this, "Product")
        .partitionKey(partitionKey)
        .build();

        

        final Map<String, String> environment = new HashMap<>();
        environment.put("PRODUCT_TABLE", productTable.getTableName());

		final Function getAllProducts = Function.Builder.create(this, "GetProductsHandler")
        .runtime(Runtime.NODEJS_12_X) 
        .code(Code.fromAsset("resources/lambda")) 
        .handler("get-all-products.getAllProductsHandler") 
        .environment(environment)
        .build();

        productTable.grantReadData(getAllProducts);

        final DynamoEventSource productEventSource = DynamoEventSource.Builder.create(productTable)
        .startingPosition(StartingPosition.LATEST)
        .batchSize(1)
        .build();

        //Function that will handle DynamoDB event sources.
        NodejsFunction sendProductEventWrapper = NodejsFunction.Builder
        .create(this, "SendProductEventHandler")
        .entry("resources/lambda/send-product-event.js") 
        .handler("handler") 
        .runtime(Runtime.NODEJS_12_X)
        .timeout(Duration.seconds(15))
        .build();

        sendProductEventWrapper.addEventSource(productEventSource);
        

        // This function will consume the 
        // Define an API Gateway REST API resource backed by our "hello" function.
        LambdaRestApi.Builder.create(this, "Endpoint")
        .handler(getAllProducts)
        .build();

        
    }
}
